
import java.io.*;
import java.nio.file.*;
import java.nio.charset.*;
import java.util.*;
import java.util.stream.*;
import java.util.function.*;
import java.lang.*;
import java.lang.ProcessBuilder.*;

import jp.kobe_u.sugar.*;
import jp.kobe_u.sugar.SugarMain;
import jp.kobe_u.sugar.SugarException;
import jp.kobe_u.sugar.csp.CSP;
import jp.kobe_u.sugar.csp.IntegerVariable;
import jp.kobe_u.sugar.converter.Converter;
import jp.kobe_u.sugar.converter.Simplifier;
import jp.kobe_u.sugar.encoder.Encoder;
import jp.kobe_u.sugar.expression.Expression;

public class CSP_SAT_clasp extends Object
{
	boolean unsat = false;

	String csp_filename = null;
	String cnf_filename = null;
	String map_filename = null;
	String out_filename = null;

	long begin_time = 0;
	long finish_time = 0;
	long elapsed_time = 0;

	public CSP_SAT_clasp(String fname)
	{
		csp_filename = fname;
		String[] no_ext_ary = fname.split("\\.", -1);
		//no_ext_ary[no_ext_ary.length-1] = "";
		//String no_ext = String.join(".", no_ext_ary);
		String no_ext = "";
		for(int i=0; i<no_ext_ary.length-1; i++)
			no_ext = no_ext + no_ext_ary[i] + ".";

		//cnf_filename = csp_filename.split("\\.", -1)[0] + ".cnf";
		//cnf_filename = no_ext + ".cnf";
		//map_filename = no_ext + ".map";
		//out_filename = no_ext + ".out";
		cnf_filename = no_ext + "cnf";
		map_filename = no_ext + "map";
		out_filename = no_ext + "out";

		System.out.println("csp_filename: " + csp_filename);
		System.out.println("cnf_filename: " + cnf_filename);
		System.out.println("map_filename: " + map_filename);
		System.out.println("out_filename: " + out_filename);

		SugarMain sugar_main = new SugarMain();
		ArrayList<Expression> exp_al = null;

		try{
			// sugar_main.encode(csp_filename, cnv_filename, map_filename);
			exp_al = new ArrayList<>(sugar_main.parse(csp_filename));
			exp_al.stream().forEach(System.out::println);
		}catch(Exception e){
			e.printStackTrace();
			System.out.println("program abort.");
			System.exit(0);
		}

		Encoder encoder = null;
		try{
			encoder = new Encoder(encodeExpression(exp_al));
			encoder.encode(cnf_filename);
			encoder.outputMap(map_filename);
			System.out.println(encoder.summary());
		}catch(Exception e){
			e.printStackTrace();
			System.out.println("program abort.");
			System.exit(0);
		}

		System.out.println("Start clasp.");
		String[] cmd = {"cmd", "/C", "clasp", "0", "-t7", cnf_filename};
		ProcessBuilder pb = new ProcessBuilder(cmd);
		begin_time = System.currentTimeMillis();
		try{
			File outfile = new File(out_filename);
			if(outfile.exists()) outfile.delete();

			pb.redirectErrorStream(true);
			pb.redirectOutput(Redirect.appendTo(outfile));

			Process p = pb.start();
			p.waitFor();
			p.destroy();
		}catch(Exception e){
			e.printStackTrace();
			System.out.println("program abort.");
			System.exit(0);
		}

		System.out.println("Get clasp results.");
		ArrayList<int[]> models = GetClaspOutput(out_filename);
		int cnt = models.size();
		final Encoder ecd = encoder;

		models.stream().forEach(model->{
			BitSet satValues = new BitSet();
			IntStream.of(model).forEach(val->{
				int i = (val<0) ? -val : val;
				if(i>0) satValues.set(i, val>0);
			});
			for(IntegerVariable v: ecd.csp.getIntegerVariables()){
				String nm = v.getName();
				if(nm.charAt(0)=='$') continue;
				v.decode(satValues);
				System.out.print("(" + nm + "," + Integer.toString(v.getValue()) + ")");
			}
			System.out.println();
		});

		if(models.size()==0){
			System.out.println("Unsatisfiable !");
		}

		finish_time = System.currentTimeMillis();
		elapsed_time = finish_time - begin_time;
		System.out.println("solve elapsed: " + Long.toString(elapsed_time) + " milli seconds.");
		System.out.println(Integer.toString(cnt) + " solutions.");
	}

	public ArrayList<int[]> GetClaspOutput(String fname)
	{
		List<String> lines = null;
		try{
		lines = Files.readAllLines(Paths.get(fname), Charset.defaultCharset());
		}catch(Exception e){
			System.out.println("In CSP_SAT_clasp::GetClaspOutput");
			e.printStackTrace();
			System.out.println("program abort.");
			System.exit(0);
		}
		if(lines==null) lines = new ArrayList<>();

		ArrayList<int[]> models = new ArrayList<>();
		ArrayList<String> tmp_al = null;
		for(String l: lines){
			String[] ss = l.split(" ", -1);
			if(ss[0].equals("s") || ss[0].equals("c")){
				if(tmp_al!=null){
					int n = tmp_al.size()-1;
					int[] ii = new int[n];
					for(int i=0; i<n; i++)
						ii[i] = Integer.parseInt(tmp_al.get(i));
					models.add(ii);
					tmp_al = null;
				}
			}
			if(ss[1].equals("Answer:"))	tmp_al = new ArrayList<>();
			if(ss[0].equals("v"))
				for(int i=1; i<ss.length; i++)
					tmp_al.add(ss[i]);
		}
		return models;
	}

	public ArrayList<int[]> GetPicosatOutput()
	{
		List<String> lines = null;
		try{
			lines = Files.readAllLines(Paths.get(out_filename), Charset.defaultCharset());
		}catch(Exception e){
			e.printStackTrace();
			System.out.println("program abort.");
			System.exit(0);
		}

		if(lines==null) lines = new ArrayList<>();

		ArrayList<int[]> models = new ArrayList<>();
		ArrayList<String> tmp_al = null;

		for(String l: lines){
			String[] ss = l.split(" ", -1);
			if(ss[0].equals("s")){
				if(tmp_al!=null){
					int n = tmp_al.size()-1;
					int[] ii = new int[n];
					for(int i=0; i<n; i++)
						ii[i] = Integer.parseInt(tmp_al.get(i));
					models.add(ii);
					tmp_al = null;
				}
			}
			if(ss[1].equals("SATISFIABLE")) tmp_al = new ArrayList<>();
			if(ss[0].equals("v"))
				for(int i=1; i<ss.length; i++)
					tmp_al.add(ss[i]);
		}

		return models;
	}

	public CSP encodeExpression(ArrayList<Expression> exp_al) throws SugarException
	{
		CSP csp = new CSP();
		Converter converter = new Converter(csp);
		converter.convert(exp_al);

		csp.propagate();
		if(csp.isUnsatisfiable()){
			unsat = true;
			return null;
		}

		// csp.simplify();
		Simplifier simplifier = new Simplifier(csp);
		simplifier.simplify();
		if(csp.isUnsatisfiable()){
			unsat = true;
			return null;
		}

		return csp;
	}

	public static void main(String... args)
	{
		CSP_SAT_clasp app = new CSP_SAT_clasp (args[0]);
	}
}

