from mip import *

prob = Model(sense=MINIMIZE, solver_name=CBC)
a = prob.add_var(name='a', var_type=CONTINUOUS)
b = prob.add_var(name='b', var_type=CONTINUOUS)
x = [0.1, 1.2, 2.05, 2.99, 3.89, 5.03, 6.01, 7.1, 7.82, 9.07]
y = [1.1, 1.9, 3.02, 4.3, 4.95, 5.97, 7.2, 7.85, 8.88, 10.1]
n = 10
zp = [prob.add_var(name='zp{}'.format(i), var_type=CONTINUOUS, lb=0) for i in range(n)]
zm = [prob.add_var(name='zm{}'.format(i), var_type=CONTINUOUS, lb=0) for i in range(n)]
for i in range(n):
  prob += y[i]-(a*x[i]+b)==zp[i]-zm[i], 'const_pm{}'.format(i)
  prob += y[i]-(a*x[i]+b)>=-zm[i], 'const_p{}'.format(i)
  prob += y[i]-(a*x[i]+b)<=zp[i], 'const_m{}'.format(i)

prob += xsum(zpi for zpi in zp) + xsum(zmi for zmi in zm)
print('objective:',prob.objective)
zp_val = np.zeros(n)
zm_val = np.zeros(n)
for i in range(n):
  print(prob.constr_by_name('const_pm{}'.format(i)))
  print(prob.constr_by_name('const_p{}'.format(i)))
  print(prob.constr_by_name('const_m{}'.format(i)))

status = prob.optimize()
print('Status', status)
print('a', prob.vars['a'].x)
print('b', prob.vars['b'].x)
print('objective:', prob.objective_value)
for i in range(n):
  zp_val[i] = prob.vars['zp{}'.format(i)].x
  zm_val[i] = prob.vars['zm{}'.format(i)].x
print(zp_val)
sse = sum(zp_val*zp_val + zm_val*zm_val)
print('sse:', sse)

'''
import numpy as np
X = np.array([[0.1,1], [1.2,1], [2.05,1], [2.99,1], [3.89,1], [5.03,1], [6.01,1], [7.1,1], [7.82,1], [9.07,1]])
y = np.array([1.1, 1.9, 3.02, 4.3, 4.95, 5.97, 7.2, 7.85, 8.88, 10.1])
XT = X.T
XTXI = np.linalg.inv(XT.dot(X))
[a,b] = XTXI.dot(XT).dot(y)
[a,b]
ab = [a,b]
e = y-X.dot(ab)
se = sum(abs(e))
sse = sum(e*e)
se
sse

'''
