from pulp import *

def make_line1(x, y, n):
    xs = []
    while x<n and y<n:
        xs.append((y,x))
        x+=1
        y+=1
    return xs

def make_line2(x, y, n):
    ys = []
    while x>=0 and y<n:
        ys.append((y,x))
        x-=1
        y+=1
    return ys

def solver(n):
    problem = LpProblem('nqueen')
    vs = []
    for y in range(n):
        vs1 = [LpVariable('v_{}_{}'.format(y,x), cat='Binary') for x in range(n)]
        vs.append(vs1)

    for y in range(n):
        problem += lpSum([vs[y][x] for x in range(n)]) == 1
        problem += lpSum([vs[x][y] for x in range(n)]) == 1

    for z in range(n):
        problem += lpSum([vs[y][x] for y, x in make_line1(z, 0, n)]) <= 1
        problem += lpSum([vs[y][x] for y, x in make_line2(z, 0, n)]) <= 1
    for z in range(1, n):
        problem += lpSum([vs[y][x] for y, x in make_line1(0, z, n)]) <= 1
        problem += lpSum([vs[y][x] for y, x in make_line2(n-1, z, n)]) <= 1

    #print(problem)

    status = problem.solve()
    print('Status', LpStatus[status])
    for y in range(n):
        for x in range(n):
            if vs[y][x].value()==1:
                print('Q', end=' ')
            else:
                print('.', end=' ')
        print("")

import time

s = time.time()
solver(4)
print(time.time()-s)

s = time.time()
solver(8)
print(time.time()-s)

s = time.time()
solver(16)
print(time.time()-s)

s = time.time()
solver(32)
print(time.time()-s)

s = time.time()
solver(100)
print(time.time()-s)


