import time as t
from pulp import *

import tkinter as  tk

def draw(size, xs):
    root = tk.Tk()
    c0 = tk.Canvas(root, width=40*size+20, height=40*size+20)
    c0.pack()
    
    for y in range(size):
        for x in range(size):
            if (y+x)%2==0:
                color = 'white'
            else:
                color = 'black'
            x1 = x*40+10
            y1 = y*40+10
            c0.create_rectangle(x1, y1, x1+40, y1+40, fill=color)

    for i in range(size**2):
        for j in range(size**2):
            if xs[i][j].value()==0: continue
            y1 = (i//size)*40+30
            x1 = (i%size)*40+30
            y2 = (j//size)*40+30
            x2 = (j%size)*40+30
            c0.create_line(x1, y1, x2, y2, fill='pink', width=3.0)
    root.mainloop()

def make_matrix(size):
    size2 = size * size
    ws = [[0] * size2 for _ in range(size2)]
    for y in range(size):
        for x in range(size):
            k = y * size + x
            for dx, dy in [(-1,-2), (1,-2), (-2,-1), (2,-1), (-2,1), (2,1), (-1,2), (1,2)]:
                x1 = x + dx
                y1 = y + dy
                if 0 <= x1 < size and 0 <= y1 < size:
                    ws[k][y1*size+x1] = 1
    return ws

def solver(size):
    size2 = size * size
    start = 0
    goal = size2-1
    ws = make_matrix(size)

    prob = LpProblem('knight-tour')

    xs = []
    for i in  range(size2):
        xs1 = [LpVariable('x{}_{}'.format(i,j), cat='Binary') for j in range(size2)]
        xs.append(xs1)

    us = [LpVariable('u{}'.format(i), cat='Integer', lowBound=0, upBound=size2-1) for i in range(size2)]

    prob += lpSum(xs[start]) == 1
    prob += lpSum([xs[i][goal] for i in range(size2)]) == 1
    for i in range(size2):
        if i==start or i==goal: continue
        prob += lpSum([xs[j][i] for j in range(size2)]) == 1
        prob += lpSum([xs[i][j] for j in range(size2)]) == 1

    for i in range(size2):
        for j in range(size2):
            if ws[i][j]==0: prob += xs[i][j] == 0

    for i in range(size2):
        for j in range(1, size2):
            if i==j: continue
            prob += us[i] + 1 - (size2 - 1) * (1 - xs[i][j]) <= us[j]
    for i in range(1, size2): 1 <= us[i]

    prob += us[0] == 0
    prob += us[size2-1] == size2 - 1
    prob += xs[0][size*2+1] == 1
    prob += us[size*2+1] == 1

    status = prob.solve()
    print('Status', LpStatus[status])
    for i in range(size):
        for j in range(size):
            k = i*size+j
            print('{:3d}'.format(int(us[k].value())), end=" ")
        print("")

    return xs


s = t.time()
ret = solver(5)
print(t.time()-s)

draw(5, ret)

s = t.time()
ret = solver(7)
print(t.time()-s)

draw(7, ret)

s = t.time()
ret = solver(9)
print(t.time()-s)

draw(9, ret)

s = t.time()
ret = solver(11)
print(t.time()-s)

draw(11, ret)

s = t.time()
ret = solver(13)
print(t.time()-s)

draw(13, ret)

