import sys, math, random
import time as t
from pulp import *
import tkinter as tk

def make_data(n):
    return [(random.randrange(480)+10, random.randrange(480)+10) for _ in range(n)]

def draw(ps, xs):
    root = tk.Tk()
    c0 = tk.Canvas(root, width=500, height=500)
    c0.pack()

    for i in range(len(xs)):
        for j in range(len(xs)):
            if xs[i][j].value()==0: continue
            c0.create_line(ps[i][0], ps[i][1], ps[j][0], ps[j][1], width=1.0)
    for i,j in ps:
        c0.create_oval(i-5, j-5, i+5, j+5, fill='green')
    root.mainloop()

def read_data():
    buff = []
    for a in sys.stdin:
        b = a.split()
        buff.append((int(b[0]), int(b[1])))
    return buff

def distance(p1, p2):
    dx = p1[0] - p2[0]
    dy = p1[1] - p2[1]
    return math.sqrt(dx*dx+dy*dy)

def make_matrix(ps):
    return [[distance(p1, p2) for p2 in ps] for p1 in ps]

def solver(ps):
    size = len(ps)
    ws = make_matrix(ps)

    prob = LpProblem('tsp')

    xs = []
    for i in range(size):
        xs1 = [LpVariable('x{}_{}'.format(i,j), cat='Binary') for j in range(size)]
        xs.append(xs1)
    us = [LpVariable('u{}'.format(i), cat='Integer', lowBound=0, upBound=size-1) for i in range(size)]

    prob += lpSum([lpDot(w, x) for w,x in zip(ws,xs)])

    for i in range(size):
        prob += lpSum([xs[j][i] for j in range(size)]) == 1
        prob += lpSum([xs[i][j] for j in range(size)]) == 1

    for i in range(size): prob += xs[i][i] == 0

    for i in range(1, size):
        for j in range(1, size):
            if i==j: continue
            #prob += us[i] + 1 - (size - 1) * (1 - xs[i][j]) <= us[j]
            prob += us[i] + 1 - size * (1 - xs[i][j]) + (size - 3) * xs[j][i] <= us[j]

    for i in range(1,size):
        prob += 1 + (1-xs[0][i]) + (size-3)*xs[i][0] <= us[i]
        prob += us[i] <= (size-1) - (1-xs[i][0]) - (size-3) * xs[0][i]


    prob += us[0] == 0
    #for i in range(1, size): prob += 1 <= us[i]

    s = t.time()
    status = prob.solve()
    print('Status', LpStatus[status])
    print('z', prob.objective.value())
    print([int(us[i].value()) for i in range(size)])
    print(t.time() - s)
	
    draw(ps, xs)

#solver(read_data())

ps30 = [(255, 352), (431, 109), (354, 238), (141, 32), (67, 351), (417, 112), (395, 108), (398, 237), (162, 230), 
        (488, 255), (464, 304), (436, 28), (47, 206), (181, 480), (424, 343), (106, 287), (429, 134), (408, 31),
        (63, 75), (429, 225), (158, 345), (307, 13), (129, 339), (120, 479), (143, 133), (92, 194), (225, 349), 
        (130, 446), (454, 156), (210, 70)]
ps31 = [(183, 411), (279, 134), (373, 16), (221, 171), (144, 21), (341, 48), (401, 260), (198, 159), (121, 245),
        (234, 411), (249, 443), (123, 53), (69, 128), (108, 439), (34, 457), (205, 315), (106, 237), (244, 302),
        (349, 238), (54, 101), (256, 450), (200, 42), (223, 219), (186, 171), (201, 414), (170, 156), (15, 216),
        (324, 194), (289, 139), (405, 400), (401, 205)]

ps32 = [(221, 82), (78, 417), (235, 434), (453, 409), (220, 341), (352, 162), (190, 460), (463, 390), (414, 428),
        (101, 195), (300, 327), (109, 365), (278, 68), (143, 141), (244, 367), (173, 67), (406, 181), (49, 313), 
        (447, 86), (146, 377), (382, 207), (226, 140), (356, 416), (374, 354), (100, 111), (455, 159), (398, 45),
        (373, 241), (133, 91), (451, 322), (50, 298), (157, 159)]

ps33 = [(17, 93), (83, 282), (52, 429), (238, 258), (81, 199), (309, 169), (207, 48), (179, 132), (169, 65), 
        (390, 366), (108, 217), (37, 160), (266, 324), (16, 219), (68, 222), (120, 430), (11, 36), (260, 454), 
        (489, 51), (44, 378), (370, 229), (362, 272), (283, 337), (191, 335), (266, 313), (475, 211), (377, 97),
        (263, 199), (479, 240), (44, 418), (75, 395), (122, 300), (428, 323)]

ps34 = [(271, 18), (448, 428), (176, 326), (432, 207), (405, 367), (116, 294), (459, 466), (103, 294), (336, 20),
        (417, 458), (130, 351), (307, 165), (86, 39), (118, 161), (213, 199), (169, 43), (361, 228), (211, 485),
        (422, 446), (69, 362), (169, 238), (356, 310), (344, 107), (99, 37), (22, 38), (325, 484), (486, 248),
        (41, 370), (478, 442), (210, 480), (473, 126), (66, 282), (129, 265), (130, 26)]

ps35 = [(341, 207), (196, 401), (15, 399), (367, 51), (30, 136), (425, 172), (278, 188), (307, 164), (263, 54),
        (220, 122), (294, 165), (257, 11), (243, 121), (144, 390), (156, 263), (49, 230), (293, 444), (477, 299),
        (377, 424), (337, 239), (197, 47), (204, 109), (435, 100), (468, 308), (55, 446), (386, 325), (483, 282),
        (38, 315), (147, 28), (398, 131), (100, 457), (437, 108), (265, 317), (366, 37), (205, 178)]

for ps in [ps30, ps31, ps32, ps33, ps34, ps35]:
    solver(ps)



